// $Id$
//
//  invoice.m
//  PDFlib
//

#import "invoice.h"

#if TARGET_OS_IPHONE
#import "PDFlib_ios/PDFlib_objc.h"
#else
#import "PDFlib/PDFlib_objc.h"
#endif

@implementation invoice

+ (NSData *) run {
      
    NSData *pdfdata=nil;

    NSString *infile = [[NSBundle mainBundle]
            pathForResource:@"stationery" ofType:@"pdf"];
    NSString *searchpath = [[NSBundle mainBundle] resourcePath];

    NSInteger           i, stationery, page, regularfont, boldfont;
    const double left = 55;
    const double right = 530;

    double      fontsize = 12, leading, y;
    double      sum, total;
    double      pagewidth = 595, pageheight = 842;
    NSString    *buf;
    NSString    *optlist;
    NSString    *baseopt =
        @"ruler        {   30 45     275   375   475} "
        "tabalignment {right left right right right} "
        "hortabmethod ruler fontsize 12 ";
    NSInteger           textflow;
    NSString    *closingtext =
        @"Terms of payment: <fillcolor={rgb 1 0 0}>30 days net. "
        "<fillcolor={gray 0}>90 days warranty starting at the day of sale. "
        "This warranty covers defects in workmanship only. "
        "<fontname=Helvetica-BoldOblique encoding=unicode>Kraxi Systems, Inc. "
        "<resetfont>will, at its option, repair or replace the "
        "product under the warranty. This warranty is not transferable. "
        "No returns or exchanges will be accepted for wet products.";

    typedef struct { char *name; double price; NSInteger quantity; } articledata;

    const articledata data[] = {
        { "Super Kite",                20,     2},
        { "Turbo Flyer",       40,     5},
        { "Giga Trash",                180,    1},
        { "Bare Bone Kit",     50,     3},
        { "Nitty Gritty",      20,     10},
        { "Pretty Dark Flyer", 75,     1},
        { "Free Gift",         0,      1},
    };

#define ARTICLECOUNT (sizeof(data)/sizeof(data[0]))

    
    /* create a new PDFlib object */
    PDFlib *pdflib = [[PDFlib alloc] init];
    if (!pdflib) {
         return nil;
    }
    
    
    @try { 
        [pdflib set_option:@"errorpolicy=exception"];

        // Set the search path for fonts and PDF files
        [pdflib set_option:[NSString stringWithFormat:@"SearchPath={{%@}}", searchpath]];
        
        [pdflib begin_document:@"" optlist:@""];

        [pdflib set_info:@"Creator" value:@"invoice_ios"];
        [pdflib set_info:@"Author" value:@"Thomas Merz"];
        [pdflib set_info:@"Title" value:@"PDFlib invoice generation demo (Objective-C)"];

        stationery = [pdflib open_pdi_document:infile optlist:@""];
        page = [pdflib open_pdi_page:stationery pagenumber:1 optlist:@""];
        boldfont = [pdflib load_font:@"Helvetica-Bold" encoding:@"unicode" optlist:@""];
        regularfont = [pdflib load_font:@"Helvetica" encoding:@"unicode" optlist:@""];
        leading = fontsize + 2;

        /* Establish coordinates with the origin in the upper left corner. */
        [pdflib begin_page_ext:pagewidth height:pageheight optlist:@"topdown"];

        [pdflib fit_pdi_page:page x:0 y:pageheight optlist:@""];
        [pdflib close_pdi_page:page];

        [pdflib setfont:regularfont fontsize:fontsize];

        /* Print the address */
        y = 170;
        [pdflib set_value:@"leading" value:leading];

        [pdflib show_xy:@"John Q. Doe" x:left y:y];
        [pdflib continue_text:@"255 Customer Lane"];
        [pdflib continue_text:@"Suite B"];
        [pdflib continue_text:@"12345 User Town"];
        [pdflib continue_text:@"Everland"];

        /* Print the header and date */

        [pdflib setfont:boldfont fontsize:fontsize];
        y = 300;
        [pdflib show_xy:@"INVOICE" x:left y:y];

        NSDate *today = [NSDate date];
        NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
        [dateFormat setDateFormat:@"MMMM d, YYYY"];
        buf = [dateFormat stringFromDate:today];

        [pdflib fit_textline:buf x:right y:y optlist:@"position {100 0}"];

        /* Print the invoice header line */
        y = 370;
        buf = @"\tITEM\tDESCRIPTION\tQUANTITY\tPRICE\tAMOUNT";

        optlist = [NSString stringWithFormat:@"%@ font %ld ", baseopt, (long)boldfont];

        textflow = [pdflib create_textflow:buf optlist:optlist];

        [pdflib fit_textflow:textflow llx:left lly:y-leading urx:right ury:y optlist:@""];
        [pdflib delete_textflow:textflow];

        /* Print the article list */

        y += 2*leading;
        total = 0;

        optlist = [NSString stringWithFormat:@"%@ font %ld ", baseopt, (long)regularfont];
        

        for (i = 0; i < (int) ARTICLECOUNT; i++) {
            sum = data[i].price * data[i].quantity;

            buf = [NSString stringWithFormat:@"\t%ld\t%@\t%ld\t%.2f\t%.2f",
                (long)i+1, @(data[i].name), (long)data[i].quantity, data[i].price, sum];

            textflow = [pdflib create_textflow:buf optlist:optlist];

            if (textflow == -1)
            {
                //printf("Error: %s\n", [pdflib get_errmsg];
                return nil;
            }

            [pdflib fit_textflow:textflow llx:left lly:y-leading urx:right ury:y optlist:@""];
            [pdflib delete_textflow:textflow];

            y += leading;
            total += sum;
        }

        y += leading;

        [pdflib setfont:boldfont fontsize:fontsize];
        optlist = [NSString stringWithFormat:@"%.2f", total];
        [pdflib fit_textline:buf x:right y:y optlist:@"position {100 0}"];

        /* Print the closing text */

        y += 5*leading;

        optlist =@"alignment=justify leading=120% fontname=Helvetica fontsize=12 encoding=unicode ";

        textflow = [pdflib create_textflow:closingtext optlist:optlist];

        if (textflow == -1)
        {
            //printf("Error: %s\n", [pdflib get_errmsg);
            return nil;
        }

        [pdflib fit_textflow:textflow llx:left lly:y + 6*leading urx:right ury:y optlist:@""];
        [pdflib delete_textflow:textflow];

        [pdflib end_page_ext:@""];
        [pdflib end_document:@""];
        [pdflib close_pdi_document:stationery];
        
        pdfdata = [pdflib get_buffer];
    } 
    @catch (PDFlibException *ex) { 
        NSString * errorMessage = 
	    [NSString stringWithFormat:@"PDFlib error %ld in '%@': %@",
            (long)[ex get_errnum], [ex get_apiname], [ex get_errmsg]];
#if TARGET_OS_IPHONE
        UIAlertView *alert = [[UIAlertView alloc] 
            initWithTitle:[ex name] message:errorMessage
            delegate:self cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert show];
#else
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setMessageText: errorMessage];
        [alert runModal];
#endif
    }
    @catch (NSException *ex) { 
#if TARGET_OS_IPHONE
        UIAlertView *alert = [[UIAlertView alloc] 
            initWithTitle:[ex name] message:[ex reason]
            delegate:self cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
        [alert show];
#else
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setMessageText: [ex reason]];
        [alert runModal];
#endif
    } 

    return pdfdata;
}

@end
